# coding: utf-8

"""
    LUSID API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


import json
import pprint
import re  # noqa: F401
from aenum import Enum, no_arg





class ReferenceListType(str, Enum):
    """
    ReferenceListType
    """

    """
    allowed enum values
    """
    PORTFOLIOGROUPIDLIST = 'PortfolioGroupIdList'
    PORTFOLIOIDLIST = 'PortfolioIdList'
    ADDRESSKEYLIST = 'AddressKeyList'
    STRINGLIST = 'StringList'
    INSTRUMENTLIST = 'InstrumentList'
    DECIMALLIST = 'DecimalList'

    @classmethod
    def from_json(cls, json_str: str) -> ReferenceListType:
        """Create an instance of ReferenceListType from a JSON string"""
        return ReferenceListType(json.loads(json_str))
