# coding: utf-8

"""
    LUSID API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from typing import Any, Dict, List, Optional
from pydantic.v1 import BaseModel, Field, StrictInt, StrictStr, conlist

class AddBusinessDaysToDateRequest(BaseModel):
    """
    AddBusinessDaysToDateRequest
    """
    business_day_offset: StrictInt = Field(..., alias="businessDayOffset")
    holiday_codes: conlist(StrictStr) = Field(..., alias="holidayCodes")
    start_date: Optional[datetime] = Field(None, alias="startDate")
    as_at: Optional[datetime] = Field(None, alias="asAt")
    __properties = ["businessDayOffset", "holidayCodes", "startDate", "asAt"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> AddBusinessDaysToDateRequest:
        """Create an instance of AddBusinessDaysToDateRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # set to None if as_at (nullable) is None
        # and __fields_set__ contains the field
        if self.as_at is None and "as_at" in self.__fields_set__:
            _dict['asAt'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> AddBusinessDaysToDateRequest:
        """Create an instance of AddBusinessDaysToDateRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return AddBusinessDaysToDateRequest.parse_obj(obj)

        _obj = AddBusinessDaysToDateRequest.parse_obj({
            "business_day_offset": obj.get("businessDayOffset"),
            "holiday_codes": obj.get("holidayCodes"),
            "start_date": obj.get("startDate"),
            "as_at": obj.get("asAt")
        })
        return _obj
